'use strict';
/**
 * Browser Sync will help us sync any changes to the browser.
 * It will also sync any events between browsers and devices.
 *
 * Read about it here: http://www.browsersync.io/
 */

var browserSync = require('browser-sync');
var config      = require('../config');

// defaults
config.merge({
    bSync: {
        ip: '127.0.0.1',
        port: 8181,
        root: 'build'
    }
});

/**
 * A configurable browsersync task that can be used with gulp
 *
 * @example
 *
 * // detailed/full example
 *
 * var bSync = require('furnace/b-sync');
 * var config = require('furnace/config');
 *
 * config.merge({
 *  bSync: {
 *      ip: '127.0.0.1',
        port: 8181,
        root: 'build'
 *  }
 * });
 *
 * gulp.task('bSync', bSync.task);
 */
exports.task = function browserSyncTask() {

    var opts      = config.load('bSync');
    var proxyHost = opts.ip + ':' + opts.port;

    browserSync({
        proxy: proxyHost
    });
};
